# encoding: utf-8

# ------------------------------------------------------------------------------
# Copyright (c) 2006 Novell, Inc. All Rights Reserved.
#
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of version 2 of the GNU General Public License as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, contact Novell, Inc.
#
# To contact Novell about this file by physical or electronic mail, you may find
# current contact information at www.novell.com.
# ------------------------------------------------------------------------------

# File:	include/novell-schematool/wizards.ycp
# Package:	Configuration of novell-schematool
# Summary:	Wizards definitions
# Authors:	Mark Clark <mclark@novell.com>
#
# $Id: wizards.ycp 27914 2006-02-13 14:32:08Z locilka $
module Yast
  module NovellSchematoolWizardsInclude
    def initialize_novell_schematool_wizards(include_target)
      Yast.import "UI"

      textdomain "novell-schematool"

      Yast.import "Sequencer"
      Yast.import "Wizard"

      Yast.include include_target, "novell-schematool/complex.rb"
      Yast.include include_target, "novell-schematool/dialogs.rb"
    end

    # Main workflow of the iprint configuration
    # @return sequence result
    def MainSequence
      aliases = { "ldapinfo" => lambda { LDAPDialog() }, "schema" => lambda do
        SchemaDialog()
      end }

      sequence = {
        "ws_start" => "ldapinfo",
        "ldapinfo" => { :abort => :abort, :next => "schema" },
        "schema"   => { :abort => :abort, :next => :next }
      }

      ret = Sequencer.Run(aliases, sequence)

      deep_copy(ret)
    end

    # Whole configuration of schema tool
    # @return sequence result
    def NovellSchematoolSequence
      aliases = {
        "read"  => [lambda { ReadDialog() }, true],
        "main"  => lambda { MainSequence() },
        "write" => [lambda { WriteDialog() }, true]
      }

      sequence = {
        "ws_start" => "read",
        "read"     => { :abort => :abort, :next => "main" },
        "main"     => { :abort => :abort, :next => "write" },
        "write"    => { :abort => :abort, :next => :next }
      }

      Wizard.CreateDialog
      Wizard.SetDesktopIcon("novell-schematool")

      ret = Sequencer.Run(aliases, sequence)

      UI.CloseDialog
      deep_copy(ret)
    end

    # Whole configuration of iprint but without reading and writing.
    # For use with autoinstallation.
    # @return sequence result
    def NovellSchematoolAutoSequence
      # Initialization dialog caption
      caption = _("Schema tool Configuration")
      # Initialization dialog contents
      contents = Label(_("Initializing..."))

      Wizard.CreateDialog
      Wizard.SetDesktopIcon("iprint")
      Wizard.SetContentsButtons(
        caption,
        contents,
        "",
        Label.BackButton,
        Label.NextButton
      )

      ret = MainSequence()

      UI.CloseDialog
      deep_copy(ret)
    end
  end
end
