# encoding: utf-8

# ------------------------------------------------------------------------------
# Copyright (c) 2006 Novell, Inc. All Rights Reserved.
#
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of version 2 of the GNU General Public License as published by the
# Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, contact Novell, Inc.
#
# To contact Novell about this file by physical or electronic mail, you may find
# current contact information at www.novell.com.
# ------------------------------------------------------------------------------

# File:	modules/NovellSchematool.ycp
# Package:	Configuration of novell-schematool
# Summary:	NovellSchematool settings, input and output functions
# Authors:	Mark Clark <mclark@novell.com>
#
# $Id: NovellSchematool.ycp 27914 2006-02-13 14:32:08Z locilka $
#
# Representation of the configuration of novell-schematool.
# Input and output routines.
require "yast"

module Yast
  class NovellSchematoolClass < Module
    def main

      textdomain "novell-schematool"

      Yast.import "NovellUtils"
      Yast.import "NovellLDAP"
      Yast.import "Popup"
      Yast.import "Progress"
      Yast.import "Report"
      Yast.import "Summary"

      @schemaLogPath = "/var/opt/novell/eDirectory/log/oes_schema.log"

      # Settings Map
      @Settings = {}

      # Data was modified?
      @modified = false


      @proposal_valid = false

      # Write only, used during autoinstallation.
      # Don't run services and SuSEconfig, it's all done at one place.
      @write_only = false

      # Abort function
      # return boolean return true if abort
      @AbortFunction = fun_ref(method(:Modified), "boolean ()")
    end

    # Abort function
    # @return [Boolean] return true if abort
    def Abort
      return @AbortFunction.call == true if @AbortFunction != nil
      false
    end

    # Data was modified?
    # @return true if modified
    def Modified
      Builtins.y2debug("modified=%1", @modified)
      @modified
    end

    # See if we are running on a build with Domain Services for Windows.
    # @return true if running on xad build.
    def RunningOnXadBuild
      ret = false

      cmd = "find / -name *.pat | grep -i novell-domain-services"
      Builtins.y2milestone(
        "NovellSchematool.RunningOnXadBuild:executing:%1",
        cmd
      )

      val = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))
      exit = Ops.get_integer(val, "exit", -1)
      if exit == 0
        ret = true
      else
        # we could have gotten a link error so look at standard out
        stdstr = Ops.get_string(val, "stdout", "")
        Builtins.y2milestone(
          "NovellShcematool.RunningOnXadBuild:search error - checking stdout:%1",
          stdstr
        )
        stdstr = "" if stdstr == nil
        if stdstr != ""
          ret = true if Builtins.issubstring(stdstr, "novell-domain-services")
        end
      end

      Builtins.y2milestone(
        "NovellSchematool.RunningOnXadBuild:returning:%1",
        ret
      )
      ret
    end

    # Get the products and schema files
    def GetSchemaData
      #  string xadEntry = "Novell Domain Services for Windows=/opt/novell/sch/msds.sch\n";
      #  boolean xadBuild = RunningOnXadBuild();

      data_file = "/opt/novell/sch/schema.conf"
      stmp = []

      # First see if the file exists.
      return nil if SCR.Read(path(".target.size"), data_file) == -1 # nil indicates empty or no file

      # Read the file into memory
      schemaData = Convert.to_string(
        SCR.Read(path(".target.string"), data_file)
      )
      if schemaData != ""
        #    if( xadBuild )
        #        schemaData = xadEntry + schemaData;

        # break up into individual lines
        stmp = Builtins.splitstring(schemaData, "\n")
      else
        stmp = nil
      end # empty or no file;

      deep_copy(stmp)
    end

    # Read all novell-schematool settings
    # @return true on success
    def Read
      # Schema read dialog caption
      caption = _("Initializing Schema Tool Configuration")

      steps = 2

      sl = 500
      Builtins.sleep(sl)

      # We do not set help text here, because it was set outside
      Progress.New(
        caption,
        " ",
        steps,
        [
          # Progress stage 1/2
          _("Read schema tool settings"),
          # Progress stage 2/2
          _("Check dependencies")
        ],
        [
          # Progress step 1/2
          _("Reading schema tool settings..."),
          # Progress step 2/2
          _("Checking dependencies..."),
          # Progress finished
          _("Finished")
        ],
        ""
      )

      # read current settings
      return false if Abort()
      Progress.NextStage

      # First, get the default LDAP/eDir data
      progress_orig = Progress.set(false)
      NovellLDAP.Settings = NovellLDAP.ReadLdapSettings
      Progress.set(progress_orig)

      # Create the ndssch log directory if it does not exist.
      cmd = "mkdir -p /var/opt/novell/eDirectory/log/"
      Builtins.y2milestone("schematool.Read:executing %1", cmd)
      mval = Convert.to_map(SCR.Execute(path(".target.bash_output"), cmd))


      # determine if Schema Tool configuration already exists
      ldap_server = ""
      admin = ""
      lport = 636
      service_configured = Convert.to_string(
        SCR.Read(path(".sysconfig.novellSchematool.SERVICE_CONFIGURED"))
      )
      if service_configured == nil || service_configured == ""
        service_configured = "no"
      end
      service_configured = Builtins.tolower(service_configured)
      if service_configured != "yes"
        # Grab any remote or local IPs from the LDAP list for defaults
        server_list = NovellLDAP.GetRemoteServers(nil)
        if Ops.greater_than(Builtins.size(server_list), 0)
          ldap_server = Ops.get_string(
            Ops.get(server_list, 0, {}),
            "CONFIG_LDAP_SERVER_IP",
            ""
          )
        else
          server_list = NovellLDAP.GetLocalServers(nil)
          ldap_server = Ops.get_string(
            Ops.get(server_list, 0, {}),
            "CONFIG_LDAP_SERVER_IP",
            ""
          )
        end
        ldap_server = "" if ldap_server == nil
        Ops.set(@Settings, "CONFIG_SCHEMATOOL_LDAP_SERVER", ldap_server)

        admin = Ops.get_string(
          NovellLDAP.Settings,
          "CONFIG_LDAP_ADMIN_CONTEXT",
          ""
        )
        admin = "" if admin == nil
        Ops.set(@Settings, "CONFIG_SCHEMATOOL_LDAP_ADMIN", admin)

        server_list = Ops.get_list(
          NovellLDAP.Settings,
          "CONFIG_LDAP_SERVERS_LIST",
          []
        )
        primary_server = NovellLDAP.GetAddressServer(
          Ops.get_string(@Settings, "CONFIG_SCHEMATOOL_LDAP_SERVER", ""),
          server_list
        )
        lport = Ops.get_integer(primary_server, "CONFIG_LDAP_SECURE_PORT", 636)
        lport = 636 if lport == nil
        Ops.set(@Settings, "CONFIG_SCHEMATOOL_LDAP_SECURE_PORT", lport)
      else
        ldap_server = Convert.to_string(
          SCR.Read(
            path(".sysconfig.novellSchematool.CONFIG_SCHEMATOOL_LDAP_SERVER")
          )
        )
        ldap_server = "" if ldap_server == nil
        Ops.set(@Settings, "CONFIG_SCHEMATOOL_LDAP_SERVER", ldap_server)
        admin = Convert.to_string(
          SCR.Read(
            path(".sysconfig.novellSchematool.CONFIG_SCHEMATOOL_LDAP_ADMIN")
          )
        )
        admin = "" if admin == nil
        Ops.set(@Settings, "CONFIG_SCHEMATOOL_LDAP_ADMIN", admin)
        lport = Builtins.tointeger(
          Convert.to_string(
            SCR.Read(
              path(
                ".sysconfig.novellSchematool.CONFIG_SCHEMATOOL_LDAP_SECURE_PORT"
              )
            )
          )
        )
        lport = 636 if lport == nil
        Ops.set(@Settings, "CONFIG_SCHEMATOOL_LDAP_SECURE_PORT", lport)
      end

      Ops.set(@Settings, "CONFIG_SCHEMATOOL_LDAP_ADMIN_PASSWORD", "") # always ask

      # Read the schema file into the list
      schema_list = GetSchemaData()
      if schema_list == nil
        err = _(
          "The Schema Extension Tool data file is either missing or empty (/opt/novell/sch/schema.conf)."
        )

        Builtins.y2milestone("NovellSchematool.write:%1", err)
        Popup.Message(err)
        return false
      end
      Builtins.y2milestone("NovellSchematool.read:schemaData: %1", schema_list)
      Ops.set(@Settings, "CONFIG_SCHEMA_DATA_FILE", schema_list)

      Builtins.y2debug("SCHEMATOOL READ Complete: %1", @Settings)

      # Error message
      Report.Error(_("Cannot read current Schema Tool settings.")) if false
      Builtins.sleep(sl)

      # check dependencies
      return false if Abort()
      Progress.NextStage

      return false if Abort()
      Progress.NextStage
      Progress.Title(_("Finished"))
      Builtins.sleep(sl)

      return false if Abort()
      @modified = false
      true
    end


    # Extend schema for the selected services.
    # @return true on success
    def ExtendSchema(selected)
      selected = deep_copy(selected)

      Builtins.y2milestone(
        "Schematool: entering ExtendSchema with selected: %1",
        selected
      )

      ldap_server = Ops.get_string(
        @Settings,
        "CONFIG_SCHEMATOOL_LDAP_SERVER",
        ""
      )
      full_admin_context = NovellUtils.Ldap2Nds(
        Ops.get_string(@Settings, "CONFIG_SCHEMATOOL_LDAP_ADMIN", "")
      )
      lport = Builtins.sformat(
        "%1",
        Ops.get_integer(@Settings, "CONFIG_SCHEMATOOL_LDAP_SECURE_PORT", 636)
      )
      admin_pwd = Ops.get_string(
        @Settings,
        "CONFIG_SCHEMATOOL_LDAP_ADMIN_PASSWORD",
        ""
      )

      ndssch_file = "/opt/novell/eDirectory/bin/ndssch"
      ldif_file = "/opt/novell/eDirectory/bin/ice"
      cmd = ""
      schema_file = ""

      error_log = "For additional infomation search for NovellSchematool in /var/log/YaST2/y2log.\n\n"
      errorStr = ""
      had_errors = false

      Builtins.foreach(selected) do |service|
        # First split the name from the path
        name_path = Builtins.splitstring(service, "=")
        raise Break if name_path == nil
        sl = 1500
        mytitle = Builtins.sformat(
          "Extend Schema for %1",
          Ops.get(name_path, 0, "")
        )
        Progress.Title(mytitle)
        paths = Builtins.splitstring(Ops.get(name_path, 1, ""), "#")
        num_paths = Builtins.size(paths)
        cnt = 0
        while Ops.less_than(cnt, num_paths)
          schema_file = Ops.get(paths, cnt, "")
          Builtins.y2milestone(
            "Schematool: schema file to extend: %1",
            schema_file
          )
          cnt = Ops.add(cnt, 1)

          tmpString = Builtins.tolower(schema_file)
          if Builtins.issubstring(tmpString, ".sch") # extending and sch file
          full_admin_context = NovellUtils.Ldap2Nds(full_admin_context)
            cmd = Ops.add(
              Ops.add(
                Ops.add(
                  Ops.add(
                    Ops.add(
                      Ops.add(
                        Ops.add(
                          Ops.add(Ops.add(ndssch_file, " -h "), ldap_server),
                          " -F '"
                        ),
                        @schemaLogPath
                      ),
                      "' '"
                    ),
                    full_admin_context
                  ),
                  "'"
                ),
                " "
              ),
              schema_file
            )
            Builtins.y2milestone("NovellSchematool:Executing %1", cmd)
            rmap = Convert.to_map(
              SCR.Execute(
                path(".target.bash_output"),
                Ops.add(Ops.add(Ops.add("ADM_PASSWD='", admin_pwd), "' "), cmd)
              )
            )
            if Ops.get_integer(rmap, "exit", -1) != 0
              errorStr = Builtins.sformat(
                "NovellSchematool:%1:failed (error=%2) to extend schema using IP address: %3, using file %4",
                Ops.get(name_path, 0, ""),
                Ops.get_integer(rmap, "exit", -1),
                ldap_server,
                schema_file
              )
              Builtins.y2milestone(errorStr)
              error_log = Ops.add(
                Ops.add(error_log, errorStr),
                "\n--------------------------------------\n"
              )
              had_errors = true
            end # extending an ldif file
          else

	    full_admin_context = NovellUtils.Nds2Ldap(full_admin_context)

            cmd = Builtins.sformat(
	      "%1 -S LDIF -f %2 -D LDAP -s %3 -p %4 -d '%5'",
              ldif_file,
              schema_file,
              ldap_server,
              lport,
              full_admin_context
            )
            Builtins.y2milestone("NovellSchematool:Executing %1", cmd)
            rmap = Convert.to_map(
              SCR.Execute(
                path(".target.bash_output"),
                Ops.add(Ops.add(Ops.add("ADM_DEST_PASSWD='", admin_pwd), "' "), cmd)
              )
            )
            if Ops.get_integer(rmap, "exit", -1) != 0
              errorStr = Builtins.sformat(
                "NovellSchematool:%1:failed (error=%2) to extend schema using IP address: %3, using file %4",
                Ops.get(name_path, 0, ""),
                Ops.get_integer(rmap, "exit", -1),
                ldap_server,
                schema_file
              )
              Builtins.y2milestone(errorStr)
              error_log = Ops.add(
                Ops.add(error_log, errorStr),
                "\n--------------------------------------\n"
              )
              had_errors = true
            end
          end
        end
        Progress.NextStep
        raise Break if Abort() #sleep(sl);
      end

      # Write out the error log if there were errors
      if had_errors
        SCR.Write(path(".target.string"), "/var/log/schematool_error", "")
        SCR.Write(
          path(".target.string"),
          "/var/log/schematool_error",
          error_log
        )
      end

      !had_errors
    end


    # Write all novell-schematool settings
    # @return true on success
    def Write
      # Schema Tool write dialog caption
      caption = _("Extending selected schema.")


      sl = 500
      Builtins.sleep(sl)


      SCR.Write(path(".sysconfig.novellSchematool.SERVICE_CONFIGURED"), "no")

      # Extend the schema that has been selected.
      ext_list = Ops.get_list(@Settings, "CONFIG_SCHEMA_SELECTED_LIST", [])

      steps = Builtins.size(ext_list)
      Progress.Simple(caption, " ", steps, "")
      Progress.Step(0)
      if ext_list != nil && Ops.greater_than(Builtins.size(ext_list), 0)
        Builtins.y2milestone("Schematool:selected list: %1", ext_list)

        if Builtins.setenv("LDAPTLS_REQCERT", "never")
          Builtins.y2milestone("LDAPTLS_REQCERT value is unset")
        else
          Builtins.y2milestone("Failed to unset TLSLDAP_REQCERT")
        end

        ret = ExtendSchema(ext_list)

        if Builtins.setenv("LDAPTLS_REQCERT", "")
          Builtins.y2milestone("LDAPTLS_REQCERT value is unset")
        else
          Builtins.y2milestone("Failed to unset TLSLDAP_REQCERT")
        end

        if ret == false
          Popup.Message(
            _(
              "There were problems with the extension of one or more schema files. See /var/log/schematool_errors."
            )
          )
        else
          Popup.Message(_("Schema extended successfully."))
        end
      else
        Builtins.y2milestone(
          "NovellSchematool.write:nothing selected to extend."
        )
      end

      # Save out the LDAP data for another session.
      SCR.Write(
        path(".sysconfig.novellSchematool.CONFIG_SCHEMATOOL_LDAP_SERVER"),
        Ops.get_string(@Settings, "CONFIG_SCHEMATOOL_LDAP_SERVER", "")
      )
      SCR.Write(
        path(".sysconfig.novellSchematool.CONFIG_SCHEMATOOL_LDAP_ADMIN"),
        Ops.get_string(@Settings, "CONFIG_SCHEMATOOL_LDAP_ADMIN", "")
      )
      SCR.Write(
        path(".sysconfig.novellSchematool.CONFIG_SCHEMATOOL_LDAP_SECURE_PORT"),
        Builtins.sformat(
          "%1",
          Ops.get_integer(@Settings, "CONFIG_SCHEMATOOL_LDAP_SECURE_PORT", 636)
        )
      )

      SCR.Write(path(".sysconfig.novellSchematool.SERVICE_CONFIGURED"), "yes")

      # Flush the sysconfig file.
      SCR.Write(path(".sysconfig.novellSchematool"), nil)

      return false if Abort()
      Progress.Finish
      true
    end

    # Get all novell-schematool settings from the first parameter
    # (For use by autoinstallation.)
    # @param [Hash] settings The YCP structure to be imported.
    # @return [Boolean] True on success
    def Import(settings)
      settings = deep_copy(settings)
      # TODO FIXME: your code here (fill the above mentioned variables)...
      true
    end

    # Dump the novell-schematool settings to a single map
    # (For use by autoinstallation.)
    # @return [Hash] Dumped settings (later acceptable by Import ())
    def Export
      # TODO FIXME: your code here (return the above mentioned variables)...
      {}
    end

    # Create a textual summary and a list of unconfigured cards
    # @return summary of the current configuration
    def Summary
      # TODO FIXME: your code here...
      # Configuration summary text for autoyast
      [_("Configuration summary..."), []]
    end

    # Create an overview table with all configured cards
    # @return table items
    def Overview
      # TODO FIXME: your code here...
      []
    end

    # Return packages needed to be installed and removed during
    # Autoinstallation to insure module has all needed software
    # installed.
    # @return [Hash] with 2 lists.
    def AutoPackages
      # TODO FIXME: your code here...
      { "install" => [], "remove" => [] }
    end

    publish :variable => :schemaLogPath, :type => "string"
    publish :variable => :Settings, :type => "map"
    publish :function => :Modified, :type => "boolean ()"
    publish :variable => :modified, :type => "boolean"
    publish :variable => :proposal_valid, :type => "boolean"
    publish :variable => :write_only, :type => "boolean"
    publish :variable => :AbortFunction, :type => "boolean ()"
    publish :function => :Abort, :type => "boolean ()"
    publish :function => :GetSchemaData, :type => "list <string> ()"
    publish :function => :Read, :type => "boolean ()"
    publish :function => :Write, :type => "boolean ()"
    publish :function => :Import, :type => "boolean (map)"
    publish :function => :Export, :type => "map ()"
    publish :function => :Summary, :type => "list ()"
    publish :function => :Overview, :type => "list ()"
    publish :function => :AutoPackages, :type => "map ()"
  end

  NovellSchematool = NovellSchematoolClass.new
  NovellSchematool.main
end
